/**
*
* Name		stpdetab -- Replace tab characters with blanks
*
* Synopsis	presult = stpdetab(ptarget,psource,pstops,incr,tarsize);
*
*		char *presult	NIL if conversion was completed; else
*				pointer to next character in psource.
*		char *ptarget	Pointer to the target string
*		char *psource	Pointer to the source string
*		char *pstops	Pointer to table of tab stops
*		int  incr	Interval separating further tab stops
*				beyond the end of pstops
*		int tarsize	The maximum size of the target string
*
* Description	This function replaces tab characters ('\t') with enough
*		blank spaces (' ') to advance to the following tab
*		position.
*
*		Tab stops are defined as follows:  The string pstop
*		consists of Ts ('t' or 'T') and other non-control
*		characters.  The Ts in pstop correspond to the positions
*		of tab stops.  The value of incr tells the interval
*		between tab stops beyond the last stop designated in
*		pstop.
*
*		Special cases:	If incr is positive and no tab stops are
*		designated in pstop, then tab stops are set every incr
*		characters starting from position 0 (i.e., the first
*		character of the line).  If incr is 0 or negative, then
*		no tab stops are set beyond those set in pstop, and any
*		additional tab characters in psource are copied intact
*		to ptarget.
*
*		Newline ('\n') characters in psource are copied to
*		ptarget.  Then the list of tab stops is used again.
*
*		STPDETAB regards all characters in psource (except tabs,
*		NULs, and newlines) as graphic characters occupying one
*		space in ptarget.
*
*		Warning:  ptarget and psource must not overlap.
*
* Examples	In these examples, column 0 means the first column in a
*		line.
*
*		To designate tab stops at column 8 and every 8 columns
*		thereafter:
*
*		    pstops = ""
*		    incr   = 8
*
*		To designate tab stops at columns 4 and 20 and every 5
*		columns thereafter:
*
*		    pstops = "0123t567890123456789t"
*		    incr   = 5
*
* Returns	presult 	Pointer to the remainder of psource if
*				there wasn't enough room in ptarget;
*				otherwise NIL.
*		ptarget 	Pointer to the altered target string.
*
* Version	3.0 (C)Copyright Blaise Computing Inc.	1986
*
**/

#include <bstring.h>

#define  BLANK	 ' '
#define  NEWLINE '\n'
#define  TAB	 '\t'

char *stpdetab(ptarget,psource,pstops,incr,tarsize)
register char *ptarget;
char	      *psource,*pstops;
int	      incr,tarsize;
{
    char c;
    register char *ptable;    /* Running pointer into pstops	      */
    register int  len;	      /* Running total of characters put into */
			      /* ptarget, also index of next char in  */
			      /* ptarget.			      */

    char *pmaxtab;	      /* Pointer to last tab stop in pstops   */
    int numspaces;
    int numbeyond;	      /* Number of spaces advanced beyond     */
			      /* pmaxtab			      */
    static int next_tab(char *,char *,int,int);

    pmaxtab = pstops;
    for (ptable = pstops; *ptable; ptable++)	       /* Set pmaxtab */
	if ((*ptable == 't') || (*ptable == 'T'))
	    pmaxtab = ptable;

    ptable = pstops;
    len = numbeyond = 0;
    tarsize--;
    while ((c = *psource) && (len < tarsize))
    {
	switch (c)
	{
	    case NEWLINE:
		*ptarget++ = NEWLINE;
		len++;
		ptable = pstops;
		numbeyond = 0;
		break;
	    case TAB:
		if (numspaces = next_tab(ptable,pmaxtab,numbeyond,incr))
		    if ((len += numspaces) < tarsize)
		    {			   /* There's enough room     */
			while (numspaces--)
			{
			    *ptarget++ = BLANK;
			    if (ptable < pmaxtab)
				ptable++;
			    else
				numbeyond++;
			}
			break;
		    }
		    else		   /* There isn't enough room */
			continue;	   /* so quit.		      */
	    /* Else TAB expansion is not in effect:		      */
	    /* just fall through and copy the TAB to ptarget.	      */
	    default:
		*ptarget++ = c;
		len++;
		if (ptable < pmaxtab)
		    ptable++;
		else
		    numbeyond++;
	}
	psource++;
    }
    *ptarget = '\0';
    return c ? psource : NIL;
}

/**
*
* Name		next_tab -- Return number of columns to next tab stop
*
* Synopsis	result = next_tab(ptable,pmaxtab,numbeyond,incr)
*		int  result	Number of columns to next tab stop
*				(always at least one if tabs expanded;
*				zero if tabs not to be expanded.)
*		char *ptable	Pointer into table of tab stops
*		char *pmaxtab	Pointer to last tab in table of tab
*				stops, or if table empty, to beginning
*				of table.
*		int  numbeyond	Number of spaces we have advanced
*				beyond pmaxtab.
*		int  incr	Interval between tab stops after the
*				last stop specified in table
*
* Definition	This function returns the number of spaces required to
*		expand a tab character.  Ptable must point to the
*		current position in the table of tab stops (e.g., if to
*		the first position if starting the line), unless we're
*		at or beyond pmaxtab, in which case numbeyond contains
*		the number of positions we've advanced beyond pmaxtab.
*
*		Zero is returned if the tab is not to be expanded.  This
*		occurs if ptable points at or beyond pmaxtab and if incr
*		is zero.
*
* Returns	presult 	Number of columns to next tab stop.
*				(always at least one if tabs expanded;
*				zero if tabs not to be expanded.)
*
**/

static int next_tab(ptable,pmaxtab,numbeyond,incr)
register char *ptable;
char	      *pmaxtab;
int	      numbeyond;
register int  incr;
{
    register int numspaces;

    if (ptable < pmaxtab)
    {				       /* Still within table	      */
	numspaces = 0;
	do		  /* Count number of columns to next tab stop */
	{
	    numspaces++;
	    ptable++;
	} while ((*ptable != 't') && (*ptable != 'T'));
	return numspaces;
    }
    else if (incr > 0)		       /* Beyond table		      */
	return (incr - (numbeyond % incr));

    return 0;			       /* No tab expansion	      */
}
